package net.minecraft.client.renderer;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferInt;
import java.awt.image.ImageObserver;

public class ImageBufferDownload implements IImageBuffer {
	private int[] imageData;
	private int imageWidth;
	private int imageHeight;

	@Override
	public BufferedImage parseUserSkin(BufferedImage image) {
		if (image == null) {
			return null;
		} else {
			this.imageWidth = 64;
			this.imageHeight = 64;
			int i = image.getWidth();
			int j = image.getHeight();
			int k;

			for (k = 1; (this.imageWidth < i) || (this.imageHeight < j); k *= 2) {
				this.imageWidth *= 2;
				this.imageHeight *= 2;
			}

			BufferedImage bufferedimage = new BufferedImage(this.imageWidth, this.imageHeight, 2);
			Graphics graphics = bufferedimage.getGraphics();
			graphics.drawImage(image, 0, 0, (ImageObserver) null);
			boolean flag = image.getHeight() == (32 * k);

			if (flag) {
				graphics.setColor(new Color(0, 0, 0, 0));
				graphics.fillRect(0 * k, 32 * k, 64 * k, 32 * k);
				graphics.drawImage(bufferedimage, 24 * k, 48 * k, 20 * k, 52 * k, 4 * k, 16 * k, 8 * k, 20 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 28 * k, 48 * k, 24 * k, 52 * k, 8 * k, 16 * k, 12 * k, 20 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 20 * k, 52 * k, 16 * k, 64 * k, 8 * k, 20 * k, 12 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 24 * k, 52 * k, 20 * k, 64 * k, 4 * k, 20 * k, 8 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 28 * k, 52 * k, 24 * k, 64 * k, 0 * k, 20 * k, 4 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 32 * k, 52 * k, 28 * k, 64 * k, 12 * k, 20 * k, 16 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 40 * k, 48 * k, 36 * k, 52 * k, 44 * k, 16 * k, 48 * k, 20 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 44 * k, 48 * k, 40 * k, 52 * k, 48 * k, 16 * k, 52 * k, 20 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 36 * k, 52 * k, 32 * k, 64 * k, 48 * k, 20 * k, 52 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 40 * k, 52 * k, 36 * k, 64 * k, 44 * k, 20 * k, 48 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 44 * k, 52 * k, 40 * k, 64 * k, 40 * k, 20 * k, 44 * k, 32 * k, (ImageObserver) null);
				graphics.drawImage(bufferedimage, 48 * k, 52 * k, 44 * k, 64 * k, 52 * k, 20 * k, 56 * k, 32 * k, (ImageObserver) null);
			}

			graphics.dispose();
			this.imageData = ((DataBufferInt) bufferedimage.getRaster().getDataBuffer()).getData();
			this.setAreaOpaque(0 * k, 0 * k, 32 * k, 16 * k);

			if (flag) {
				this.setAreaTransparent(32 * k, 0 * k, 64 * k, 32 * k);
			}

			this.setAreaOpaque(0 * k, 16 * k, 64 * k, 32 * k);
			this.setAreaOpaque(16 * k, 48 * k, 48 * k, 64 * k);
			return bufferedimage;
		}
	}

	@Override
	public void skinAvailable() {
	}

	private void setAreaTransparent(int p_setAreaTransparent_1_, int p_setAreaTransparent_2_, int p_setAreaTransparent_3_, int p_setAreaTransparent_4_) {
		for (int i = p_setAreaTransparent_1_; i < p_setAreaTransparent_3_; ++i) {
			for (int j = p_setAreaTransparent_2_; j < p_setAreaTransparent_4_; ++j) {
				int k = this.imageData[i + (j * this.imageWidth)];

				if (((k >> 24) & 255) < 128) { return; }
			}
		}

		for (int l = p_setAreaTransparent_1_; l < p_setAreaTransparent_3_; ++l) {
			for (int i1 = p_setAreaTransparent_2_; i1 < p_setAreaTransparent_4_; ++i1) {
				this.imageData[l + (i1 * this.imageWidth)] &= 16777215;
			}
		}
	}

	/**
	 * Makes the given area of the image opaque
	 */
	private void setAreaOpaque(int x, int y, int width, int height) {
		for (int i = x; i < width; ++i) {
			for (int j = y; j < height; ++j) {
				this.imageData[i + (j * this.imageWidth)] |= -16777216;
			}
		}
	}
}
