package net.minecraft.client.renderer.block.model;

import java.lang.reflect.Type;

import javax.annotation.Nullable;

import com.google.gson.*;

import net.minecraft.util.EnumFacing;
import net.minecraft.util.JsonUtils;

public class BlockPartFace {
	public static final EnumFacing FACING_DEFAULT = null;
	public final EnumFacing cullFace;
	public final int tintIndex;
	public final String texture;
	public final BlockFaceUV blockFaceUV;

	public BlockPartFace(@Nullable EnumFacing cullFaceIn, int tintIndexIn, String textureIn, BlockFaceUV blockFaceUVIn) {
		this.cullFace = cullFaceIn;
		this.tintIndex = tintIndexIn;
		this.texture = textureIn;
		this.blockFaceUV = blockFaceUVIn;
	}

	static class Deserializer implements JsonDeserializer<BlockPartFace> {
		@Override
		public BlockPartFace deserialize(JsonElement p_deserialize_1_, Type p_deserialize_2_, JsonDeserializationContext p_deserialize_3_) throws JsonParseException {
			JsonObject jsonobject = p_deserialize_1_.getAsJsonObject();
			EnumFacing enumfacing = this.parseCullFace(jsonobject);
			int i = this.parseTintIndex(jsonobject);
			String s = this.parseTexture(jsonobject);
			BlockFaceUV blockfaceuv = (BlockFaceUV) p_deserialize_3_.deserialize(jsonobject, BlockFaceUV.class);
			return new BlockPartFace(enumfacing, i, s, blockfaceuv);
		}

		protected int parseTintIndex(JsonObject object) {
			return JsonUtils.getInt(object, "tintindex", -1);
		}

		private String parseTexture(JsonObject object) {
			return JsonUtils.getString(object, "texture");
		}

		@Nullable
		private EnumFacing parseCullFace(JsonObject object) {
			String s = JsonUtils.getString(object, "cullface", "");
			return EnumFacing.byName(s);
		}
	}
}
