package net.minecraft.client.renderer.texture;

import java.awt.image.BufferedImage;
import java.io.IOException;

import org.apache.commons.io.IOUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.minecraft.client.resources.IResource;
import net.minecraft.client.resources.IResourceManager;
import net.minecraft.client.resources.data.TextureMetadataSection;
import net.minecraft.src.Config;
import net.minecraft.util.ResourceLocation;
import shadersmod.client.ShadersTex;

public class SimpleTexture extends AbstractTexture {
	private static final Logger LOG = LogManager.getLogger();
	protected final ResourceLocation textureLocation;

	public SimpleTexture(ResourceLocation textureResourceLocation) {
		this.textureLocation = textureResourceLocation;
	}

	@Override
	public void loadTexture(IResourceManager resourceManager) throws IOException {
		this.deleteGlTexture();
		IResource iresource = null;

		try {
			iresource = resourceManager.getResource(this.textureLocation);
			BufferedImage bufferedimage = TextureUtil.readBufferedImage(iresource.getInputStream());
			boolean flag = false;
			boolean flag1 = false;

			if (iresource.hasMetadata()) {
				try {
					TextureMetadataSection texturemetadatasection = (TextureMetadataSection) iresource.getMetadata("texture");

					if (texturemetadatasection != null) {
						flag = texturemetadatasection.getTextureBlur();
						flag1 = texturemetadatasection.getTextureClamp();
					}
				} catch (RuntimeException runtimeexception) {
					LOG.warn("Failed reading metadata of: {}", new Object[] { this.textureLocation, runtimeexception });
				}
			}

			if (Config.isShaders()) {
				ShadersTex.loadSimpleTexture(this.getGlTextureId(), bufferedimage, flag, flag1, resourceManager, this.textureLocation, this.getMultiTexID());
			} else {
				TextureUtil.uploadTextureImageAllocate(this.getGlTextureId(), bufferedimage, flag, flag1);
			}
		} finally {
			IOUtils.closeQuietly(iresource);
		}
	}
}
