package net.minecraft.entity.boss.dragon.phase;

import javax.annotation.Nullable;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import net.minecraft.entity.boss.EntityDragon;
import net.minecraft.util.math.Vec3d;

public class PhaseChargingPlayer extends PhaseBase {
	private static final Logger LOGGER = LogManager.getLogger();
	private Vec3d targetLocation;
	private int timeSinceCharge;

	public PhaseChargingPlayer(EntityDragon dragonIn) {
		super(dragonIn);
	}

	/**
	 * Gives the phase a chance to update its status. Called by dragon's onLivingUpdate. Only used when !worldObj.isRemote.
	 */
	@Override
	public void doLocalUpdate() {
		if (this.targetLocation == null) {
			LOGGER.warn("Aborting charge player as no target was set.");
			this.dragon.getPhaseManager().setPhase(PhaseList.HOLDING_PATTERN);
		} else if ((this.timeSinceCharge > 0) && (this.timeSinceCharge++ >= 10)) {
			this.dragon.getPhaseManager().setPhase(PhaseList.HOLDING_PATTERN);
		} else {
			double d0 = this.targetLocation.squareDistanceTo(this.dragon.posX, this.dragon.posY, this.dragon.posZ);

			if ((d0 < 100.0D) || (d0 > 22500.0D) || this.dragon.isCollidedHorizontally || this.dragon.isCollidedVertically) {
				++this.timeSinceCharge;
			}
		}
	}

	/**
	 * Called when this phase is set to active
	 */
	@Override
	public void initPhase() {
		this.targetLocation = null;
		this.timeSinceCharge = 0;
	}

	public void setTarget(Vec3d p_188668_1_) {
		this.targetLocation = p_188668_1_;
	}

	/**
	 * Returns the maximum amount dragon may rise or fall during this phase
	 */
	@Override
	public float getMaxRiseOrFall() {
		return 3.0F;
	}

	@Override
	@Nullable

	/**
	 * Returns the location the dragon is flying toward
	 */
	public Vec3d getTargetLocation() {
		return this.targetLocation;
	}

	@Override
	public PhaseList<PhaseChargingPlayer> getPhaseList() {
		return PhaseList.CHARGING_PLAYER;
	}
}
