package net.minecraft.item;

import javax.annotation.Nullable;

import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.EntityHanging;
import net.minecraft.entity.item.EntityItemFrame;
import net.minecraft.entity.item.EntityPainting;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

public class ItemHangingEntity extends Item {
	private final Class<? extends EntityHanging> hangingEntityClass;

	public ItemHangingEntity(Class<? extends EntityHanging> entityClass) {
		this.hangingEntityClass = entityClass;
		this.setCreativeTab(CreativeTabs.DECORATIONS);
	}

	/**
	 * Called when a Block is right-clicked with this Item
	 */
	@Override
	public EnumActionResult onItemUse(ItemStack stack, EntityPlayer playerIn, World worldIn, BlockPos pos, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
		BlockPos blockpos = pos.offset(facing);

		if ((facing != EnumFacing.DOWN) && (facing != EnumFacing.UP) && playerIn.canPlayerEdit(blockpos, facing, stack)) {
			EntityHanging entityhanging = this.createEntity(worldIn, blockpos, facing);

			if ((entityhanging != null) && entityhanging.onValidSurface()) {
				if (!worldIn.isRemote) {
					entityhanging.playPlaceSound();
					worldIn.spawnEntityInWorld(entityhanging);
				}

				--stack.stackSize;
			}

			return EnumActionResult.SUCCESS;
		} else {
			return EnumActionResult.FAIL;
		}
	}

	@Nullable
	private EntityHanging createEntity(World worldIn, BlockPos pos, EnumFacing clickedSide) {
		return this.hangingEntityClass == EntityPainting.class ? new EntityPainting(worldIn, pos, clickedSide)
				: (this.hangingEntityClass == EntityItemFrame.class ? new EntityItemFrame(worldIn, pos, clickedSide) : null);
	}
}
