package net.minecraft.item;

import java.util.List;
import java.util.Map;

import javax.annotation.Nullable;

import com.google.common.collect.Maps;

import net.minecraft.block.BlockJukebox;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.stats.StatList;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.translation.I18n;
import net.minecraft.world.World;

public class ItemRecord extends Item {
	private static final Map<SoundEvent, ItemRecord> RECORDS = Maps.<SoundEvent, ItemRecord> newHashMap();
	private final SoundEvent sound;
	private final String displayName;

	protected ItemRecord(String p_i46742_1_, SoundEvent soundIn) {
		this.displayName = "item.record." + p_i46742_1_ + ".desc";
		this.sound = soundIn;
		this.maxStackSize = 1;
		this.setCreativeTab(CreativeTabs.MISC);
		RECORDS.put(this.sound, this);
	}

	/**
	 * Called when a Block is right-clicked with this Item
	 */
	@Override
	public EnumActionResult onItemUse(ItemStack stack, EntityPlayer playerIn, World worldIn, BlockPos pos, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ) {
		IBlockState iblockstate = worldIn.getBlockState(pos);

		if ((iblockstate.getBlock() == Blocks.JUKEBOX) && !iblockstate.getValue(BlockJukebox.HAS_RECORD).booleanValue()) {
			if (!worldIn.isRemote) {
				((BlockJukebox) Blocks.JUKEBOX).insertRecord(worldIn, pos, iblockstate, stack);
				worldIn.playEvent((EntityPlayer) null, 1010, pos, Item.getIdFromItem(this));
				--stack.stackSize;
				playerIn.addStat(StatList.RECORD_PLAYED);
			}

			return EnumActionResult.SUCCESS;
		} else {
			return EnumActionResult.PASS;
		}
	}

	/**
	 * allows items to add custom lines of information to the mouseover description
	 */
	@Override
	public void addInformation(ItemStack stack, EntityPlayer playerIn, List<String> tooltip, boolean advanced) {
		tooltip.add(this.getRecordNameLocal());
	}

	public String getRecordNameLocal() {
		return I18n.translateToLocal(this.displayName);
	}

	/**
	 * Return an item rarity from EnumRarity
	 */
	@Override
	public EnumRarity getRarity(ItemStack stack) {
		return EnumRarity.RARE;
	}

	@Nullable
	public static ItemRecord getBySound(SoundEvent soundIn) {
		return RECORDS.get(soundIn);
	}

	public SoundEvent getSound() {
		return this.sound;
	}
}
