package net.minecraft.network.play.client;

import java.io.IOException;

import net.minecraft.network.Packet;
import net.minecraft.network.PacketBuffer;
import net.minecraft.network.play.INetHandlerPlayServer;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;

public class CPacketPlayerDigging implements Packet<INetHandlerPlayServer> {
	private BlockPos position;
	private EnumFacing facing;

	/** Status of the digging (started, ongoing, broken). */
	private CPacketPlayerDigging.Action action;

	public CPacketPlayerDigging() {
	}

	public CPacketPlayerDigging(CPacketPlayerDigging.Action actionIn, BlockPos posIn, EnumFacing facingIn) {
		this.action = actionIn;
		this.position = posIn;
		this.facing = facingIn;
	}

	/**
	 * Reads the raw packet data from the data stream.
	 */
	@Override
	public void readPacketData(PacketBuffer buf) throws IOException {
		this.action = buf.readEnumValue(CPacketPlayerDigging.Action.class);
		this.position = buf.readBlockPos();
		this.facing = EnumFacing.getFront(buf.readUnsignedByte());
	}

	/**
	 * Writes the raw packet data to the data stream.
	 */
	@Override
	public void writePacketData(PacketBuffer buf) throws IOException {
		buf.writeEnumValue(this.action);
		buf.writeBlockPos(this.position);
		buf.writeByte(this.facing.getIndex());
	}

	/**
	 * Passes this Packet on to the NetHandler for processing.
	 */
	@Override
	public void processPacket(INetHandlerPlayServer handler) {
		handler.processPlayerDigging(this);
	}

	public BlockPos getPosition() {
		return this.position;
	}

	public EnumFacing getFacing() {
		return this.facing;
	}

	public CPacketPlayerDigging.Action getAction() {
		return this.action;
	}

	public static enum Action {
		START_DESTROY_BLOCK, ABORT_DESTROY_BLOCK, STOP_DESTROY_BLOCK, DROP_ALL_ITEMS, DROP_ITEM, RELEASE_USE_ITEM, SWAP_HELD_ITEMS;
	}
}
