package net.minecraft.tileentity;

import java.util.Random;

import javax.annotation.Nullable;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.WorldServer;
import net.minecraft.world.storage.loot.ILootContainer;
import net.minecraft.world.storage.loot.LootContext;
import net.minecraft.world.storage.loot.LootTable;

public abstract class TileEntityLockableLoot extends TileEntityLockable implements ILootContainer {
	protected ResourceLocation lootTable;
	protected long lootTableSeed;

	protected boolean checkLootAndRead(NBTTagCompound compound) {
		if (compound.hasKey("LootTable", 8)) {
			this.lootTable = new ResourceLocation(compound.getString("LootTable"));
			this.lootTableSeed = compound.getLong("LootTableSeed");
			return true;
		} else {
			return false;
		}
	}

	protected boolean checkLootAndWrite(NBTTagCompound compound) {
		if (this.lootTable != null) {
			compound.setString("LootTable", this.lootTable.toString());

			if (this.lootTableSeed != 0L) {
				compound.setLong("LootTableSeed", this.lootTableSeed);
			}

			return true;
		} else {
			return false;
		}
	}

	protected void fillWithLoot(@Nullable EntityPlayer player) {
		if (this.lootTable != null) {
			LootTable loottable = this.worldObj.getLootTableManager().getLootTableFromLocation(this.lootTable);
			this.lootTable = null;
			Random random;

			if (this.lootTableSeed == 0L) {
				random = new Random();
			} else {
				random = new Random(this.lootTableSeed);
			}

			LootContext.Builder lootcontext$builder = new LootContext.Builder((WorldServer) this.worldObj);

			if (player != null) {
				lootcontext$builder.withLuck(player.getLuck());
			}

			loottable.fillInventory(this, random, lootcontext$builder.build());
		}
	}

	@Override
	public ResourceLocation getLootTable() {
		return this.lootTable;
	}

	public void func_189404_a(ResourceLocation p_189404_1_, long p_189404_2_) {
		this.lootTable = p_189404_1_;
		this.lootTableSeed = p_189404_2_;
	}
}
