package us.dev.direkt.module.internal.misc.styledchat.internal.modes;

import com.google.common.collect.ImmutableMap;
import us.dev.direkt.module.internal.misc.styledchat.internal.AbstractStyledChatMode;

import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Foundry
 */
public class ScriptMode extends AbstractStyledChatMode {
    private static final Map<String, String> characterReplacementMap = new ImmutableMap.Builder<String, String>()
            .put(Pattern.quote("A"), Matcher.quoteReplacement(Character.toString((char)0x0e04)))
            .put(Pattern.quote("B"), Matcher.quoteReplacement(Character.toString((char)0x0e52)))
            .put(Pattern.quote("C"), Matcher.quoteReplacement(Character.toString((char)0x03c2)))
            .put(Pattern.quote("D"), Matcher.quoteReplacement(Character.toString((char)0x0e54)))
            .put(Pattern.quote("E"), Matcher.quoteReplacement(Character.toString((char)0x0454)))
            .put(Pattern.quote("F"), Matcher.quoteReplacement(Character.toString((char)0x0166)))
            .put(Pattern.quote("G"), Matcher.quoteReplacement(Character.toString((char)0xfeee)))
            .put(Pattern.quote("H"), Matcher.quoteReplacement(Character.toString((char)0x0452)))
            .put(Pattern.quote("I"), Matcher.quoteReplacement(Character.toString((char)0x0e40)))
            .put(Pattern.quote("J"), Matcher.quoteReplacement(Character.toString((char)0x05df)))
            .put(Pattern.quote("K"), Matcher.quoteReplacement(Character.toString((char)0x043a)))
            .put(Pattern.quote("L"), Matcher.quoteReplacement(Character.toString((char)0x026d)))
            .put(Pattern.quote("M"), Matcher.quoteReplacement(Character.toString((char)0x0e53)))
            .put(Pattern.quote("N"), Matcher.quoteReplacement(Character.toString((char)0x0e20)))
            .put(Pattern.quote("O"), Matcher.quoteReplacement(Character.toString((char)0x0e4f)))
            .put(Pattern.quote("P"), Matcher.quoteReplacement(Character.toString((char)0x05e7)))
            .put(Pattern.quote("Q"), Matcher.quoteReplacement(Character.toString((char)0x1ee3)))
            .put(Pattern.quote("R"), Matcher.quoteReplacement(Character.toString((char)0x0433)))
            .put(Pattern.quote("S"), Matcher.quoteReplacement(Character.toString((char)0x0e23)))
            .put(Pattern.quote("T"), Matcher.quoteReplacement(Character.toString((char)0x0547)))
            .put(Pattern.quote("U"), Matcher.quoteReplacement(Character.toString((char)0x0e22)))
            .put(Pattern.quote("V"), Matcher.quoteReplacement(Character.toString((char)0x05e9)))
            .put(Pattern.quote("W"), Matcher.quoteReplacement(Character.toString((char)0x0e2c)))
            .put(Pattern.quote("X"), Matcher.quoteReplacement(Character.toString((char)0x05d0)))
            .put(Pattern.quote("Y"), Matcher.quoteReplacement(Character.toString((char)0x05e5)))
            .put(Pattern.quote("Z"), Matcher.quoteReplacement(Character.toString((char)0x0579)))
            .put(Pattern.quote("a"), Matcher.quoteReplacement(Character.toString((char)0x0e04)))
            .put(Pattern.quote("b"), Matcher.quoteReplacement(Character.toString((char)0x0e52)))
            .put(Pattern.quote("c"), Matcher.quoteReplacement(Character.toString((char)0x03c2)))
            .put(Pattern.quote("d"), Matcher.quoteReplacement(Character.toString((char)0x0e54)))
            .put(Pattern.quote("e"), Matcher.quoteReplacement(Character.toString((char)0x0454)))
            .put(Pattern.quote("f"), Matcher.quoteReplacement(Character.toString((char)0x0166)))
            .put(Pattern.quote("g"), Matcher.quoteReplacement(Character.toString((char)0xfeee)))
            .put(Pattern.quote("h"), Matcher.quoteReplacement(Character.toString((char)0x0452)))
            .put(Pattern.quote("i"), Matcher.quoteReplacement(Character.toString((char)0x0e40)))
            .put(Pattern.quote("j"), Matcher.quoteReplacement(Character.toString((char)0x05df)))
            .put(Pattern.quote("k"), Matcher.quoteReplacement(Character.toString((char)0x043a)))
            .put(Pattern.quote("l"), Matcher.quoteReplacement(Character.toString((char)0x026d)))
            .put(Pattern.quote("m"), Matcher.quoteReplacement(Character.toString((char)0x0e53)))
            .put(Pattern.quote("n"), Matcher.quoteReplacement(Character.toString((char)0x0e20)))
            .put(Pattern.quote("o"), Matcher.quoteReplacement(Character.toString((char)0x0e4f)))
            .put(Pattern.quote("p"), Matcher.quoteReplacement(Character.toString((char)0x05e7)))
            .put(Pattern.quote("q"), Matcher.quoteReplacement(Character.toString((char)0x1ee3)))
            .put(Pattern.quote("r"), Matcher.quoteReplacement(Character.toString((char)0x0433)))
            .put(Pattern.quote("s"), Matcher.quoteReplacement(Character.toString((char)0x0e23)))
            .put(Pattern.quote("t"), Matcher.quoteReplacement(Character.toString((char)0x0547)))
            .put(Pattern.quote("u"), Matcher.quoteReplacement(Character.toString((char)0x0e22)))
            .put(Pattern.quote("v"), Matcher.quoteReplacement(Character.toString((char)0x05e9)))
            .put(Pattern.quote("w"), Matcher.quoteReplacement(Character.toString((char)0x0e2c)))
            .put(Pattern.quote("x"), Matcher.quoteReplacement(Character.toString((char)0x05d0)))
            .put(Pattern.quote("y"), Matcher.quoteReplacement(Character.toString((char)0x05e5)))
            .put(Pattern.quote("z"), Matcher.quoteReplacement(Character.toString((char)0x0579)))
            .put(Pattern.quote("1"), Matcher.quoteReplacement(Character.toString((char)0x0031)))
            .put(Pattern.quote("2"), Matcher.quoteReplacement(Character.toString((char)0x0032)))
            .put(Pattern.quote("3"), Matcher.quoteReplacement(Character.toString((char)0x0033)))
            .put(Pattern.quote("4"), Matcher.quoteReplacement(Character.toString((char)0x0034)))
            .put(Pattern.quote("5"), Matcher.quoteReplacement(Character.toString((char)0x0035)))
            .put(Pattern.quote("6"), Matcher.quoteReplacement(Character.toString((char)0x0036)))
            .put(Pattern.quote("7"), Matcher.quoteReplacement(Character.toString((char)0x0037)))
            .put(Pattern.quote("8"), Matcher.quoteReplacement(Character.toString((char)0x0038)))
            .put(Pattern.quote("9"), Matcher.quoteReplacement(Character.toString((char)0x0039)))
            .put(Pattern.quote("0"), Matcher.quoteReplacement(Character.toString((char)0x0030)))
            .put(Pattern.quote("-"), Matcher.quoteReplacement(Character.toString((char)0x002d)))
            .put(Pattern.quote("="), Matcher.quoteReplacement(Character.toString((char)0x003d)))
            .put(Pattern.quote("~"), Matcher.quoteReplacement(Character.toString((char)0x007e)))
            .put(Pattern.quote("!"), Matcher.quoteReplacement(Character.toString((char)0x0021)))
            .put(Pattern.quote("@"), Matcher.quoteReplacement(Character.toString((char)0x0040)))
            .put(Pattern.quote("#"), Matcher.quoteReplacement(Character.toString((char)0x0023)))
            .put(Pattern.quote("$"), Matcher.quoteReplacement(Character.toString((char)0x0024)))
            .put(Pattern.quote("%"), Matcher.quoteReplacement(Character.toString((char)0x0025)))
            .put(Pattern.quote("^"), Matcher.quoteReplacement(Character.toString((char)0x005e)))
            .put(Pattern.quote("&"), Matcher.quoteReplacement(Character.toString((char)0x0026)))
            .put(Pattern.quote("*"), Matcher.quoteReplacement(Character.toString((char)0x002a)))
            .put(Pattern.quote("("), Matcher.quoteReplacement(Character.toString((char)0x0028)))
            .put(Pattern.quote(")"), Matcher.quoteReplacement(Character.toString((char)0x0029)))
            .put(Pattern.quote("_"), Matcher.quoteReplacement(Character.toString((char)0x005f)))
            .put(Pattern.quote("+"), Matcher.quoteReplacement(Character.toString((char)0x002b)))
            .put(Pattern.quote(","), Matcher.quoteReplacement(Character.toString((char)0x002c)))
            .put(Pattern.quote("<"), Matcher.quoteReplacement(Character.toString((char)0x003c)))
            .put(Pattern.quote("."), Matcher.quoteReplacement(Character.toString((char)0x002e)))
            .put(Pattern.quote("/"), Matcher.quoteReplacement(Character.toString((char)0x002f)))
            .put(Pattern.quote("?"), Matcher.quoteReplacement(Character.toString((char)0x003f)))
            .put(Pattern.quote(";"), Matcher.quoteReplacement(Character.toString((char)0x003b)))
            .put(Pattern.quote(":"), Matcher.quoteReplacement(Character.toString((char)0x003a)))
            .put(Pattern.quote("'"), Matcher.quoteReplacement(Character.toString((char)0x0027)))
            .put(Pattern.quote("\""), Matcher.quoteReplacement(Character.toString((char)0x0022)))
            .put(Pattern.quote("["), Matcher.quoteReplacement(Character.toString((char)0x005b)))
            .put(Pattern.quote("]"), Matcher.quoteReplacement(Character.toString((char)0x005d)))
            .put(Pattern.quote("{"), Matcher.quoteReplacement(Character.toString((char)0x007b)))
            .put(Pattern.quote("}"), Matcher.quoteReplacement(Character.toString((char)0x007d)))
            .put(Pattern.quote("\\"), Matcher.quoteReplacement(Character.toString((char)0x005c)))
            .put(Pattern.quote("|"), Matcher.quoteReplacement(Character.toString((char)0x007c)))
            .build();

    public ScriptMode() {
        super("Script", characterReplacementMap);
    }
}
